#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['beni']

package_data = \
{'': ['*']}

install_requires = \
['pyyaml', 'typeguard', 'packaging', 'tqdm', 'flit-core >=2,<4']

extras_require = \
{'dev': ['black', 'ipython', 'mypy'], 'test': ['pytest']}

entry_points = \
{'console_scripts': ['beni = beni:main']}

setup(name='beni',
      version='0.4.1',
      description='Generate environment.yml from pyproject.toml',
      author='Saul Shanabrook',
      author_email='s.shanabrook@gmail.com',
      url='https://github.com/quansight-labs/beni',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      entry_points=entry_points,
      python_requires='>=3.8',
     )
